@@ -205,8 +205,6 @@ class GtkUI(glchess.ui.UI):
         self.__saveGameDialogs = {}
         self.__joinGameDialogs = []
         
-        self._initOpenGL()
-
         # Set the message panel to the tooltip style
         # (copied from Gedit)
         # In Gtk+ 2.11+ (I think) tip_window is now private so skip if it's not there (bug #459740)
@@ -261,8 +259,6 @@ class GtkUI(glchess.ui.UI):
         if haveGnomeSupport:
             self._gui.get_widget('menu_help').show()
         
-        self.saveDialog = dialogs.SaveDialog(self)
-        
         self._updateViewButtons()
         
         # Watch for config changes
@@ -273,20 +269,6 @@ class GtkUI(glchess.ui.UI):
                     'move_format', 'promotion_type', 'board_view',
                     'enable_networking']:
             glchess.config.watch(key, self.__applyConfig)
-            
-    def _initOpenGL(self):
-        # Configure openGL
-        try:
-            gtk.gdkgl
-        except AttributeError:
-            self.glConfig = None
-        else:
-            display_mode = (gtk.gdkgl.MODE_RGB | gtk.gdkgl.MODE_DEPTH | gtk.gdkgl.MODE_DOUBLE)
-            try:
-                self.glConfig = gtk.gdkgl.Config(mode = display_mode)
-            except gtk.gdkgl.NoMatches:
-                display_mode &= ~gtk.gdkgl.MODE_DOUBLE
-                self.glConfig = gtk.gdkgl.Config(mode = display_mode)
 
     # Public methods
     
@@ -355,7 +337,8 @@ class GtkUI(glchess.ui.UI):
         """Extends ui.UI"""
         moveFormat = glchess.config.get('move_format')
         showComments = glchess.config.get('show_comments')
-        self.view = chessview.GtkView(self, title, feedback, moveFormat = moveFormat, showComments = showComments)
+        self.view = chessview.GtkView(self, feedback, moveFormat = moveFormat, showComments = showComments)
+        self.view.setTitle(title)
         self.view.isPlayable = isPlayable
         self.view.viewWidget.setRenderGL(self.__renderGL)
         viewport = self.__getWidget('game_viewport')
@@ -363,12 +346,6 @@ class GtkUI(glchess.ui.UI):
         if child is not None:
             viewport.remove(child)
         viewport.add(self.view.widget)
-        
-        # Set the window title to the name of the game
-        title = _('Chess')
-        if self.view is not None and len(self.view.title) > 0:
-            title += " - %s" % self.view.title
-        self._gui.get_widget('glchess_app').set_title(title)
 
         # Set toolbar/menu buttons to state for this game
         self._updateViewButtons()
@@ -379,6 +356,22 @@ class GtkUI(glchess.ui.UI):
 
         return self.view
 
+    def updateTitle(self):
+        """
+        """
+        # Set the window title to the name of the game
+        if self.view is not None and len(self.view.title) > 0:
+            if self.view.needsSaving:
+                # Translators: This is the window title when playing a game that needs saving
+                title = _('Chess - *%(game_name)s') % {'game_name': self.view.title}
+            else:
+                # Translators: This is the window title when playing a game that is saved
+                title = _('Chess - %(game_name)s') % {'game_name': self.view.title}
+        else:
+            # Translators: This is the window title when not playing a game
+            title = _('Chess')            
+        self._gui.get_widget('glchess_app').set_title(title)
+
     def addLogWindow(self, title, executable, description):
         """
         """
@@ -466,7 +459,26 @@ class GtkUI(glchess.ui.UI):
         # Update the open dialogs
         for dialog in self.__joinGameDialogs:
             dialog.removeNetworkGame(game)
-        
+            
+    def requestSave(self, title):
+        """Extends glchess.ui.UI"""
+        dialog = gtk.MessageDialog(flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+                                   type = gtk.MESSAGE_WARNING,
+                                   message_format = title)
+        dialog.format_secondary_text("If you don't save the changes to this game will be permanently lost")
+        dialog.add_button(_('Close _without saving'), gtk.RESPONSE_OK)
+        dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT)
+        dialog.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT)
+
+        response = dialog.run()
+        dialog.destroy()
+        if response == gtk.RESPONSE_ACCEPT:
+            return glchess.ui.SAVE_YES
+        elif response == gtk.RESPONSE_OK:
+            return glchess.ui.SAVE_NO
+        else:
+            return glchess.ui.SAVE_ABORT
+
     def close(self):
         """Extends glchess.ui.UI"""
         # Save the window size
@@ -574,7 +586,17 @@ class GtkUI(glchess.ui.UI):
         # Enable/disable OpenGL rendering
         elif name == 'show_3d':            
             if value and not chessview.haveGLSupport:
-                self._gui.get_widget('3d_support_dialog').show()
+                title = _('Unable to enable 3D mode')
+                errors = '\n'.join(chessview.openGLErrors)
+                description = _("""You are unable to play in 3D mode due to the following problems:
+%(errors)s
+
+Please contact your system administrator to resolve these problems, until then you will be able to play chess in 2D mode.""" % {'errors': errors})
+                dialog = gtk.MessageDialog(type = gtk.MESSAGE_WARNING, message_format = title)
+                dialog.format_secondary_text(description)
+                dialog.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
+                dialog.run()
+                dialog.destroy()
                 glchess.config.set('show_3d', False)
                 value = False
             self.__renderGL = value
@@ -649,7 +671,7 @@ class GtkUI(glchess.ui.UI):
         widget = self._gui.get_widget(name)
         assert(widget is not None), 'Unable to find widget: %s' % name
         return widget
-    
+
     def _on_white_time_paint(self, widget, event):
         """Gtk+ callback"""
         self.__drawTime(self.whiteTimeString, widget, (0.0, 0.0, 0.0), (1.0, 1.0, 1.0))
@@ -747,11 +769,8 @@ class GtkUI(glchess.ui.UI):
         # Get the move number
         moveNumber = model.get_value(iter, 1)
         
-        string = 'Showing move number %d ' % moveNumber
         if moveNumber == len(model) - 1:
-            string += ' (latest)'
             moveNumber = -1
-        print string
 
         # Disable buttons when at the end
         haveMoves = len(model) > 1
@@ -947,25 +966,6 @@ class GtkUI(glchess.ui.UI):
         
         # Stop the event - the window will be closed by the menu event
         return True
-   
-    def _on_save_dialog_response(self, widget, responseId):
-        """Gtk+ callback"""
-        self.saveDialog.setVisible(False)
-        
-        if responseId != gtk.RESPONSE_OK:
-            return
-        
-        # Save the requested games
-        for (view, save, _) in self.saveDialog.model:
-            if save:
-                view.feedback.save()
-
-        self.feedback.onQuit()
-        
-    def _on_save_dialog_delete(self, widget, event):
-        """Gtk+ callback"""
-        # Leave it to use to hide the dialog
-        return True
     
     def _on_resize(self, widget, event):
         """Gtk+ callback"""
@@ -991,24 +991,12 @@ class GtkUI(glchess.ui.UI):
 
     def _on_close_window(self, widget, event):
         """Gtk+ callback"""
-        self._quit()
+        self.feedback.onQuit()
         return True
         
     def _on_menu_quit(self, widget):
         """Gtk+ callback"""
-        self._quit()
-        
-    def _quit(self):
-        # Check if any views need saving
-        viewsToSave = []
-        if self.view.feedback.needsSaving():
-            viewsToSave.append(self.view)
-
-        if len(viewsToSave) == 0:
-            self.feedback.onQuit()
-        else:
-            self.saveDialog.setViews(viewsToSave)
-            self.saveDialog.setVisible(True)
+        self.feedback.onQuit()
 
 if __name__ == '__main__':
     ui = GtkUI()
