@@ -12,6 +12,8 @@
  * pancho@nuclecu.unam.mx
  */
 
+#include <config.h>
+
 #include <sys/types.h>
 #include <sys/time.h>
 #include <string.h>
@@ -20,25 +22,33 @@
 #include <dirent.h>
 #include <config.h>
 
-#include <gtk/gtk.h>
-#include <gnome.h>
 #include <glib/gi18n.h>
-#include <gconf/gconf-client.h>
+#include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <games-clock.h>
-#include <games-gconf.h>
-#include <games-frame.h>
-#include <games-stock.h>
-#include <games-scores.h>
-#include <games-scores-dialog.h>
 
+#include <libgames-support/games-clock.h>
+#include <libgames-support/games-conf.h>
+#include <libgames-support/games-frame.h>
+#include <libgames-support/games-stock.h>
+#include <libgames-support/games-scores.h>
+#include <libgames-support/games-scores-dialog.h>
+
+#ifdef HAVE_GNOME
+#include <gnome.h>
+#endif /* HAVE_GNOME */
+    
 #include "mahjongg.h"
 #include "drawing.h"
 #include "solubility.h"
 #include "maps.h"
 
 #define APPNAME "mahjongg"
-#define APPNAME_LONG "Mahjongg"
+#define APPNAME_LONG N_("Mahjongg")
+
+#define KEY_TILESET       "tileset"
+#define KEY_SHOW_TOOLBAR  "show_toolbar"
+#define KEY_BGCOLOUR      "bgcolour"
+#define KEY_MAPSET        "mapset"
 
 /* #defines for the tile selection code. */
 #define SELECTED_FLAG   1
@@ -47,6 +57,12 @@
 /* The number of half-cycles to blink during a hint (less 1) */
 #define HINT_BLINK_NUM 5
 
+#define DEFAULT_WIDTH 530
+#define DEFAULT_HEIGHT 440
+
+#define DEFAULT_TILESET "default.png"
+#define DEFAULT_MAPSET "Easy"
+
 tilepos *pos = 0;
 
 tile tiles[MAX_TILES];
@@ -69,8 +85,6 @@ GtkAction *leavefullscreen_action;
 gint selected_tile, visible_tiles;
 gint sequence_number;
 
-static gint windowwidth, windowheight;
-
 GList *tileset_list = NULL;
 
 gchar *tileset = NULL;
@@ -92,7 +106,6 @@ GtkWidget *chrono;
 gint paused = 0;
 
 /* for the preferences */
-GConfClient *conf_client;
 GtkWidget *warn_cb = NULL, *confirm_cb = NULL;
 GtkWidget *colour_well = NULL;
 GtkWidget *pref_dialog = NULL;
@@ -102,6 +115,8 @@ gboolean new_map = TRUE;
 
 GamesScores *highscores;
 
+static gint get_mapset_index (void);
+
 static const GamesScoresDescription scoredesc = { NULL,
   "Easy",
   "mahjongg",
@@ -172,14 +187,16 @@ fullscreen_callback (GtkAction * action)
     gtk_window_unfullscreen (GTK_WINDOW (window));
 }
 
-static void
+static gboolean
 window_state_callback (GtkWidget * widget, GdkEventWindowState * event)
 {
   if (!(event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN))
-    return;
+    return FALSE;
 
   set_fullscreen_actions (event->new_window_state &
 			  GDK_WINDOW_STATE_FULLSCREEN);
+    
+  return FALSE;
 }
 
 /* At the end of the game, hint, shuffle and pause all become unavailable. */
@@ -210,14 +227,14 @@ update_menu_sensitivities (void)
    * shown properly, due to a bug in GtkToolButton in Gtk+. 
    * See Bug #332573 for a description of the workaround.
    */
-  /*gtk_widget_queue_draw (toolbar);
-   * gtk_widget_queue_resize (toolbar);
-   * while (gtk_events_pending ())
-   * gtk_main_iteration ();
-   * gtk_widget_queue_draw (toolbar);
-   * gtk_widget_queue_resize (toolbar);
-   * while (gtk_events_pending ())
-   * gtk_main_iteration (); */
+  gtk_widget_queue_draw (toolbar);
+  gtk_widget_queue_resize (toolbar);
+  while (gtk_events_pending ())
+    gtk_main_iteration ();
+  gtk_widget_queue_draw (toolbar);
+  gtk_widget_queue_resize (toolbar);
+  while (gtk_events_pending ())
+    gtk_main_iteration ();
 
 }
 
@@ -247,19 +264,23 @@ tileset_callback (GtkWidget * widget, void *data)
   entry = g_list_nth (tileset_list,
 		      gtk_combo_box_get_active (GTK_COMBO_BOX (widget)));
 
-  gconf_client_set_string (conf_client,
-			   "/apps/mahjongg/tileset", entry->data, NULL);
+  games_conf_set_string (NULL, KEY_TILESET, entry->data);
 }
 
 static void
-tileset_changed_cb (GConfClient * client,
-		    guint cnxn_id, GConfEntry * entry, gpointer user_data)
+conf_value_changed_cb (GamesConf *conf,
+                       const char *group,
+                       const char *key,
+                       gpointer user_data)
 {
-  char *tile_tmp = NULL;
+  if (group != NULL)
+    return;
+
+  if (strcmp (key, KEY_TILESET) == 0) {
+    char *tile_tmp;
+
+    tile_tmp = games_conf_get_string_with_default (NULL, KEY_TILESET, DEFAULT_TILESET);
 
-  tile_tmp = gconf_client_get_string (conf_client,
-				      "/apps/mahjongg/tileset", NULL);
-  if (tile_tmp) {
     if (strcmp (tile_tmp, selected_tileset) != 0) {
       g_free (selected_tileset);
       selected_tileset = tile_tmp;
@@ -268,25 +289,57 @@ tileset_changed_cb (GConfClient * client,
     } else {
       g_free (tile_tmp);
     }
+  } else if (strcmp (key, KEY_SHOW_TOOLBAR) == 0) {
+    gboolean state;
+
+    state = games_conf_get_boolean (NULL, KEY_SHOW_TOOLBAR, NULL);
+
+    if (state)
+      gtk_widget_show (toolbar);
+    else
+      gtk_widget_hide (toolbar);
+  } else if (strcmp (key, KEY_BGCOLOUR) == 0) {
+    gchar *colour;
+
+    colour = games_conf_get_string (NULL, KEY_BGCOLOUR, NULL);
+    set_background (colour);
+    if (colour_well != NULL) {
+      gtk_color_button_set_color (GTK_COLOR_BUTTON (colour_well), &bgcolour);
+    }
+    draw_all_tiles ();
+  } else if (strcmp (key, KEY_MAPSET) == 0) {
+    GtkWidget *dialog;
+    gint response;
+
+    mapset = get_mapset_index ();
+
+    new_map = TRUE;
+
+    /* Skip the dialog if a game isn't in play. */
+    if (game_over || !games_clock_get_seconds (GAMES_CLOCK (chrono))) {
+      new_game ();
+      return;
+    }
+
+    dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                                                 GTK_DIALOG_MODAL,
+                                                 GTK_MESSAGE_QUESTION,
+                                                 GTK_BUTTONS_NONE,
+                                                _("Do you want to finish the current game or start playing with the new map immediately?"));
+    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+                                               _("If you choose to finish with the old map then the next game will use the new map."));
+    gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+                            _("_Finish"), GTK_RESPONSE_REJECT,
+                            GTK_STOCK_NEW, GTK_RESPONSE_ACCEPT,
+                            NULL);
+    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
+    response = gtk_dialog_run (GTK_DIALOG (dialog));
+    if (response == GTK_RESPONSE_ACCEPT)
+      new_game ();
+    gtk_widget_destroy (dialog);
   }
 }
 
-static void
-show_toolbar_changed_cb (GConfClient * client,
-			 guint cnxn_id,
-			 GConfEntry * entry, gpointer user_data)
-{
-  gboolean state;
-
-  state = gconf_client_get_bool (conf_client,
-				 "/apps/mahjongg/show_toolbar", NULL);
-
-  if (state)
-    gtk_widget_show (toolbar);
-  else
-    gtk_widget_hide (toolbar);
-}
-
 static void
 show_tb_callback (void)
 {
@@ -295,37 +348,21 @@ show_tb_callback (void)
   state =
     gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (show_toolbar_action));
 
-  gconf_client_set_bool (conf_client, "/apps/mahjongg/show_toolbar",
-			 state, NULL);
-}
-
-static void
-bg_colour_changed_cb (GConfClient * client,
-		      guint cnxn_id, GConfEntry * entry, gpointer user_data)
-{
-  gchar *colour;
-
-  colour = gconf_client_get_string (conf_client,
-				    "/apps/mahjongg/bgcolour", NULL);
-  set_background (colour);
-  if (colour_well != NULL) {
-    gtk_color_button_set_color (GTK_COLOR_BUTTON (colour_well), &bgcolour);
-  }
-  draw_all_tiles ();
+  games_conf_set_boolean (NULL, KEY_SHOW_TOOLBAR, state);
 }
 
 static void
 bg_colour_callback (GtkWidget * widget, gpointer data)
 {
-  static char *tmp = "";
   GdkColor colour;
+  char str[64];
 
   gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colour);
 
-  tmp = g_strdup_printf ("#%04x%04x%04x", colour.red,
-			 colour.green, colour.blue);
+  g_snprintf (str, sizeof (str), "#%04x%04x%04x",
+              colour.red, colour.green, colour.blue);
 
-  gconf_client_set_string (conf_client, "/apps/mahjongg/bgcolour", tmp, NULL);
+  games_conf_set_string (NULL, KEY_BGCOLOUR, str);
 }
 
 static gint
@@ -335,8 +372,7 @@ get_mapset_index (void)
   gint newmapset = -1;
   gint i;
 
-  mapset_name = gconf_client_get_string (conf_client,
-					 "/apps/mahjongg/mapset", NULL);
+  mapset_name = games_conf_get_string_with_default (NULL, KEY_MAPSET, DEFAULT_MAPSET);
   for (i = 0; i < nmaps; i++)
     if (g_utf8_collate (mapset_name, maps[i].name) == 0)
       newmapset = i;
@@ -353,43 +389,6 @@ get_mapset_index (void)
   return newmapset;
 }
 
-static void
-mapset_changed_cb (GConfClient * client,
-		   guint cnxn_id, GConfEntry * entry, gpointer user_data)
-{
-  GtkWidget *dialog;
-  gint response;
-
-
-  mapset = get_mapset_index ();
-
-  new_map = TRUE;
-
-  /* Skip the dialog if a game isn't in play. */
-  if (game_over || !games_clock_get_seconds (GAMES_CLOCK (chrono))) {
-    new_game ();
-    return;
-  }
-
-  dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (window),
-					       GTK_DIALOG_MODAL,
-					       GTK_MESSAGE_QUESTION,
-					       GTK_BUTTONS_NONE,
-					       "<b>%s</b>\n\n%s",
-					       _
-					       ("Do you want to finish the current game or start playing with the new map immediately?"),
-					       _
-					       ("If you choose to finish with the old map then the next game will use the new map."));
-  gtk_dialog_add_buttons (GTK_DIALOG (dialog), _("_Finish"),
-			  GTK_RESPONSE_REJECT, GTK_STOCK_NEW,
-			  GTK_RESPONSE_ACCEPT, NULL);
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
-  response = gtk_dialog_run (GTK_DIALOG (dialog));
-  if (response == GTK_RESPONSE_ACCEPT)
-    new_game ();
-  gtk_widget_destroy (dialog);
-}
-
 static void
 set_map_selection (GtkWidget * widget, void *data)
 {
@@ -397,29 +396,14 @@ set_map_selection (GtkWidget * widget, void *data)
 
   target_mapset = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
 
-  gconf_client_set_string (conf_client,
-			   "/apps/mahjongg/mapset",
-			   maps[target_mapset].name, NULL);
+  games_conf_set_string (NULL, KEY_MAPSET, maps[target_mapset].name);
 }
 
 static void
 init_config (void)
 {
-  gconf_client_add_dir (conf_client,
-			"/apps/mahjongg", GCONF_CLIENT_PRELOAD_ONELEVEL,
-			NULL);
-  gconf_client_notify_add (conf_client,
-			   "/apps/mahjongg/show_toolbar",
-			   show_toolbar_changed_cb, NULL, NULL, NULL);
-  gconf_client_notify_add (conf_client,
-			   "/apps/mahjongg/bgcolour",
-			   bg_colour_changed_cb, NULL, NULL, NULL);
-  gconf_client_notify_add (conf_client,
-			   "/apps/mahjongg/mapset",
-			   mapset_changed_cb, NULL, NULL, NULL);
-  gconf_client_notify_add (conf_client,
-			   "/apps/mahjongg/tileset",
-			   tileset_changed_cb, NULL, NULL, NULL);
+  g_signal_connect (games_conf_get_default (), "value-changed",
+                    G_CALLBACK (conf_value_changed_cb), NULL);
 }
 
 static void
@@ -565,7 +549,7 @@ tile_event (gint tileno, gint button)
 }
 
 static void
-fill_tile_menu (GtkWidget * menu, gchar * sdir)
+fill_tile_menu (GtkWidget * menu)
 {
   struct dirent *e;
   DIR *dir;
@@ -579,8 +563,7 @@ fill_tile_menu (GtkWidget * menu, gchar * sdir)
 
   tileset_list = NULL;
 
-  dname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-				     (sdir), FALSE, NULL);
+  dname = g_build_filename (PIXMAPDIR, NULL);
   dir = opendir (dname);
 
   if (!dir) {
@@ -599,8 +582,13 @@ fill_tile_menu (GtkWidget * menu, gchar * sdir)
       g_free (s);
       continue;
     }
+      
+    // TODO: Strip off suffix (bug #406856)
+
+    gchar** tileset_name = g_strsplit (s, ".", -1);
+    gtk_combo_box_append_text (GTK_COMBO_BOX (menu), tileset_name[0]);
+    g_strfreev (tileset_name);
 
-    gtk_combo_box_append_text (GTK_COMBO_BOX (menu), s);
     tileset_list = g_list_append (tileset_list, s);
     if (!strcmp (tileset, s)) {
       gtk_combo_box_set_active (GTK_COMBO_BOX (menu), itemno);
@@ -690,7 +678,7 @@ you_won (void)
     if (dialog) {
       gtk_window_present (GTK_WINDOW (dialog));
     } else {
-      dialog = games_scores_dialog_new (highscores, _("Mahjongg Scores"));
+      dialog = games_scores_dialog_new (GTK_WINDOW (window), highscores, _("Mahjongg Scores"));
       games_scores_dialog_set_category_description (GAMES_SCORES_DIALOG
 						    (dialog), _("Map:"));
       message =
@@ -765,7 +753,7 @@ properties_callback (void)
 		    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);
 
   omenu = gtk_combo_box_new_text ();
-  fill_tile_menu (omenu, "mahjongg");
+  fill_tile_menu (omenu);
   g_signal_connect (G_OBJECT (omenu), "changed",
 		    G_CALLBACK (tileset_callback), NULL);
   gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 0, 1);
@@ -944,18 +932,25 @@ about_callback (void)
   gchar *license = games_get_license (_("Mahjongg"));
 
   gtk_show_about_dialog (GTK_WINDOW (window),
-			 "name", _("Mahjongg"),
+#if GTK_CHECK_VERSION (2, 11, 0)
+                         "program-name", _("Mahjongg"),
+#else
+                         "name", _("Mahjongg"),
+#endif
 			 "version", VERSION,
 			 "comments",
-			 _("A matching game played with Mahjongg tiles."),
-			 "copyright",
-			 "Copyright \xc2\xa9 1998-2005 Free Software Foundation, Inc.",
-			 "license", license, "authors", authors, "artists",
-			 artists, "documenters", documenters,
-			 "translator_credits", _("translator-credits"),
-			 "logo-icon-name", "gnome-mahjongg", "website",
-			 "http://www.gnome.org/projects/gnome-games/",
-			 "wrap-license", TRUE, NULL);
+			 _("A matching game played with Mahjongg tiles.\n\nMahjongg is a part of GNOME Games."),
+			 "copyright", "Copyright \xc2\xa9 1998-2007 Free Software Foundation, Inc.",
+			 "license", license,
+                         "wrap-license", TRUE,
+                         "authors", authors,
+                         "artists", artists,
+                         "documenters", documenters,
+			 "translator-credits", _("translator-credits"),
+			 "logo-icon-name", "gnome-mahjongg",
+                         "website", "http://www.gnome.org/projects/gnome-games",
+                         "website-label", _("GNOME Games web site"),
+                         NULL);
   g_free (license);
 }
 
@@ -1002,7 +997,7 @@ scores_callback (GtkAction * action, gpointer data)
   if (dialog) {
     gtk_window_present (GTK_WINDOW (dialog));
   } else {
-    dialog = games_scores_dialog_new (highscores, _("Mahjongg Scores"));
+    dialog = games_scores_dialog_new (GTK_WINDOW (window), highscores, _("Mahjongg Scores"));
     games_scores_dialog_set_category_description (GAMES_SCORES_DIALOG
 						  (dialog), _("Map:"));
   }
@@ -1188,17 +1183,6 @@ do_game (void)
   generate_game (g_random_int ());	/* puts in the positions of the tiles */
 }
 
-/* Record any changes to our window size. */
-static gboolean
-window_configure_cb (GtkWidget * w, GdkEventConfigure * e, gpointer data)
-{
-  gconf_client_set_int (conf_client, "/apps/mahjongg/width", e->width, NULL);
-  gconf_client_set_int (conf_client, "/apps/mahjongg/height",
-			e->height, NULL);
-
-  return FALSE;
-}
-
 static void
 load_preferences (void)
 {
@@ -1206,28 +1190,11 @@ load_preferences (void)
 
   mapset = get_mapset_index ();
 
-  buf = gconf_client_get_string (conf_client,
-				 "/apps/mahjongg/bgcolour", NULL);
+  buf = games_conf_get_string (NULL, KEY_BGCOLOUR, NULL);
   set_background (buf);
   g_free (buf);
 
-  buf = gconf_client_get_string (conf_client, "/apps/mahjongg/tileset", NULL);
-  if (buf == NULL) {
-    buf = g_strdup ("default.png");
-  }
-  selected_tileset = g_strdup (buf);
-  g_free (buf);
-
-  windowwidth = gconf_client_get_int (conf_client,
-				      "/apps/mahjongg/width", NULL);
-  if (windowwidth <= 0)
-    windowwidth = 530;
-
-  windowheight = gconf_client_get_int (conf_client,
-				       "/apps/mahjongg/height", NULL);
-  if (windowheight <= 0)
-    windowheight = 440;
-
+  selected_tileset = games_conf_get_string_with_default (NULL, KEY_TILESET, DEFAULT_TILESET);
 
   load_images (selected_tileset);
 }
@@ -1296,7 +1263,11 @@ shuffle_tiles_callback (void)
 static void
 help_cb (GtkAction * action, gpointer data)
 {
+#ifdef HAVE_GNOME
   gnome_help_display ("mahjongg.xml", NULL, NULL);
+#else
+#warning need to port help to gtk-only!
+#endif /* HAVE_GNOME */
 }
 
 static const GtkActionEntry actions[] = {
@@ -1373,7 +1344,9 @@ static const char ui_description[] =
   "    <toolitem action='UndoMove'/>"
   "    <toolitem action='RedoMove'/>"
   "    <toolitem action='Hint'/>"
-  "    <toolitem action='LeaveFullscreen'/>" "  </toolbar>" "</ui>";
+  "    <toolitem action='LeaveFullscreen'/>"
+  "  </toolbar>"
+  "</ui>";
 
 
 static void
@@ -1408,9 +1381,7 @@ create_menus (GtkUIManager * ui_manager)
   set_fullscreen_actions (FALSE);
 
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (show_toolbar_action),
-				gconf_client_get_bool (conf_client,
-						       "/apps/mahjongg/show_toolbar",
-						       NULL));
+                                games_conf_get_boolean (NULL, KEY_SHOW_TOOLBAR, NULL));
 }
 
 static void
@@ -1439,7 +1410,22 @@ main (int argc, char *argv[])
   GtkUIManager *ui_manager;
   GtkAccelGroup *accel_group;
 
+  GOptionContext *context;
+
+#ifdef HAVE_GNOME
+  GnomeClient *client;
   GnomeProgram *program;
+#else
+  gboolean retval;
+  GError *error = NULL;
+#endif
+
+#if defined(HAVE_GNOME) || defined(HAVE_RSVG_GNOMEVFS)
+  /* If we're going to use gnome-vfs, we need to init threads before
+   * calling any glib functions.
+   */
+  g_thread_init (NULL);
+#endif
 
   setgid_io_init ();
 
@@ -1447,10 +1433,39 @@ main (int argc, char *argv[])
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
+  context = g_option_context_new (NULL);
+#if GLIB_CHECK_VERSION (2, 12, 0)
+  g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
+#endif
+
+#ifdef HAVE_GNOME
   program = gnome_program_init (APPNAME, VERSION,
 				LIBGNOMEUI_MODULE,
 				argc, argv,
-				GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
+                                GNOME_PARAM_GOPTION_CONTEXT, context,
+				GNOME_PARAM_APP_DATADIR, DATADIR,
+                                NULL);
+#else
+  g_option_context_add_group (context, gtk_get_option_group (TRUE));
+
+  retval = g_option_context_parse (context, &argc, &argv, &error);
+  g_option_context_free (context);
+  if (!retval) {
+    g_print ("%s", error->message);
+    g_error_free (error);
+    exit (1);
+  }
+#endif /* HAVE_GNOME */
+
+  g_set_application_name (_(APPNAME_LONG));
+
+  if (!games_conf_initialise (APPNAME)) {
+    /* Set the defaults */
+    games_conf_set_boolean (NULL, KEY_SHOW_TOOLBAR, TRUE);
+    games_conf_set_string (NULL, KEY_TILESET, "postmodern.svg");
+    games_conf_set_string (NULL, KEY_MAPSET, "Easy");
+    games_conf_set_string (NULL, KEY_BGCOLOUR, "#34385b");
+  }
 
   games_stock_init ();
 
@@ -1459,21 +1474,15 @@ main (int argc, char *argv[])
   load_maps ();
   init_scores ();
 
-  conf_client = gconf_client_get_default ();
-  if (!games_gconf_sanity_check_string
-      (conf_client, "/apps/mahjongg/tileset")) {
-    return 1;
-  }
+  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title (GTK_WINDOW (window), _(APPNAME_LONG));
 
-  window = gnome_app_new (APPNAME, _(APPNAME_LONG));
+  gtk_window_set_default_size (GTK_WINDOW (window), DEFAULT_WIDTH, DEFAULT_HEIGHT);
+  games_conf_add_window (GTK_WINDOW (window), NULL);
 
   load_preferences ();
 
-  gtk_window_set_default_size (GTK_WINDOW (window), windowwidth,
-			       windowheight);
-  g_signal_connect (G_OBJECT (window), "configure_event",
-		    G_CALLBACK (window_configure_cb), NULL);
-  g_signal_connect (G_OBJECT (window), "window_state_event",
+  g_signal_connect (window, "window-state-event",
 		    G_CALLBACK (window_state_callback), NULL);
 
   /* Statusbar for a chrono, Tiles left and Moves left */
@@ -1534,7 +1543,7 @@ main (int argc, char *argv[])
   gtk_box_pack_end (GTK_BOX (statusbar), status_box, FALSE, FALSE, 0);
   gtk_box_pack_end (GTK_BOX (vbox), statusbar, FALSE, FALSE, 0);
 
-  gnome_app_set_contents (GNOME_APP (window), vbox);
+  gtk_container_add (GTK_CONTAINER (window), vbox);
 
   /* FIXME: get these in the best place (as per the comment below. */
   init_config ();
@@ -1548,17 +1557,18 @@ main (int argc, char *argv[])
 
   gtk_widget_show_all (window);
 
-  if (!gconf_client_get_bool
-      (conf_client, "/apps/mahjongg/show_toolbar", NULL))
+  if (!games_conf_get_boolean (NULL, KEY_SHOW_TOOLBAR, NULL))
     gtk_widget_hide (toolbar);
 
   message_flash (_("Remove matching pairs of tiles."));
 
   gtk_main ();
 
-  gnome_accelerators_sync ();
+  games_conf_shutdown ();
 
+#ifdef HAVE_GNOME
   g_object_unref (program);
+#endif /* HAVE_GNOME */
 
   return 0;
 }
