@@ -6,16 +6,20 @@
  *
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <libgnome/gnome-help.h>
 
-#include <games-files.h>
-#include <games-gridframe.h>
-#include <games-scores-dialog.h>
-#include <games-stock.h>
+#ifdef HAVE_GNOME
+#include <libgnome/gnome-help.h>
+#endif /* HAVE_GNOME */
+
+#include <libgames-support/games-conf.h>
+#include <libgames-support/games-files.h>
+#include <libgames-support/games-gridframe.h>
+#include <libgames-support/games-scores-dialog.h>
+#include <libgames-support/games-stock.h>
 
 #include "same-gnome.h"
 
@@ -24,6 +28,9 @@
 #include "input.h"
 #include "ui.h"
 
+#define DEFAULT_WINDOW_WIDTH 450
+#define DEFAULT_WINDOW_HEIGHT 350
+
 /* Define an alternative to ngettext if we don't have it. Of course it isn't
  * a proper substitute for ngettext, but it is the best we can do. */
 #ifndef HAVE_NGETTEXT
@@ -116,19 +123,25 @@ about_cb (GtkWidget * widget)
 
 
   gtk_show_about_dialog (GTK_WINDOW (application),
+#if GTK_CHECK_VERSION (2, 11, 0)
+                         "program-name", _(APPNAME_LONG),
+#else
+                         "name", _(APPNAME_LONG),
+#endif
 			 "authors", authors,
 			 "documenters", documenters,
 			 "comments",
 			 _
-			 ("I want to play that game! You know, they all go whirly-round and you click on them and they vanish!"),
-			 "copyright",
-			 "Copyright \xc2\xa9 2007 Callum McKenzie", "license",
-			 license, "name", _(APPNAME_LONG),
-			 "translator_credits", _("translator-credits"),
+			 ("I want to play that game! You know, they all go whirly-round and you click on them and they vanish!\n\nSame GNOME is a part of GNOME Games."),
+			 "copyright", "Copyright \xc2\xa9 2007 Callum McKenzie",
+                         "license", license,
+                         "wrap-license", TRUE,
+			 "translator-credits", _("translator-credits"),
 			 "version", VERSION,
 			 "logo-icon-name", "gnome-samegnome",
-			 "website", "http://www.gnome.org/projects/gnome-games/",
-			 "wrap-license", TRUE, NULL);
+			 "website", "http://www.gnome.org/projects/gnome-games",
+                         "website-label", _("GNOME Games web site"),
+                         NULL);
   g_free (license);
 }
 
@@ -430,7 +443,11 @@ size_change_cb (GtkRadioAction * action, gpointer data)
 static void
 help_cb (void)
 {
+#ifdef HAVE_GNOME
   gnome_help_display ("same-gnome.xml", NULL, NULL);
+#else
+#warning need to port help to gtk-only!
+#endif /* HAVE_GNOME */
 }
 
 /* FIXME: Will we ever want this ? */
@@ -442,18 +459,7 @@ custom_size_cb (void)
 }
 #endif
 
-static gboolean
-window_resize_cb (GtkWidget * window, GdkEventConfigure * event)
-{
-  gconf_client_set_int (gcclient, GCONF_WINDOW_WIDTH_KEY, event->width, NULL);
-  gconf_client_set_int (gcclient, GCONF_WINDOW_HEIGHT_KEY, event->height,
-			NULL);
-
-  return FALSE;
-}
-
-
-const GtkActionEntry actions[] = {
+static const GtkActionEntry actions[] = {
   {"GameMenu", NULL, N_("_Game")},
   {"ViewMenu", NULL, N_("_View")},
   {"SizeMenu", NULL, N_("_Size")},
@@ -487,7 +493,7 @@ const GtkToggleActionEntry toggle_actions[] = {
    G_CALLBACK (animation_cb)}
 };
 
-const char ui_description[] =
+static const char ui_description[] =
   "<ui>"
   "  <menubar name='MainMenu'>"
   "    <menu action='GameMenu'>"
@@ -512,7 +518,10 @@ const char ui_description[] =
   "    </menu>"
   "    <menu action='HelpMenu'>"
   "      <menuitem action='Contents'/>"
-  "      <menuitem action='About'/>" "    </menu>" "  </menubar>" "</ui>";
+  "      <menuitem action='About'/>"
+  "    </menu>"
+  "  </menubar>"
+  "</ui>";
 
 void
 build_gui (void)
@@ -527,12 +536,12 @@ build_gui (void)
 
   application = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title (GTK_WINDOW (application), _(APPNAME_LONG));
-  gtk_window_set_default_size (GTK_WINDOW (application), window_width,
-			       window_height);
+  gtk_window_set_default_size (GTK_WINDOW (application),
+                               DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT);
+  games_conf_add_window (GTK_WINDOW (application), NULL);
+
   g_signal_connect (G_OBJECT (application), "delete_event",
 		    G_CALLBACK (quit_cb), NULL);
-  g_signal_connect (G_OBJECT (application), "configure_event",
-		    G_CALLBACK (window_resize_cb), NULL);
   g_signal_connect (G_OBJECT (application), "window_state_event",
 		    G_CALLBACK (window_state_cb), NULL);
 
@@ -562,9 +571,7 @@ build_gui (void)
   set_fullscreen_actions (FALSE);
 
   gtk_toggle_action_set_active (animation_action,
-				gconf_client_get_bool (gcclient,
-						       GCONF_FAST_ANIMATION_KEY,
-						       NULL));
+                                games_conf_get_boolean (NULL, KEY_FAST_ANIMATION, NULL));
 
   ui_manager = gtk_ui_manager_new ();
   gtk_ui_manager_insert_action_group (ui_manager, action_group, 1);
@@ -605,16 +612,15 @@ build_gui (void)
   gtk_box_pack_start (GTK_BOX (vbox), gtk_hseparator_new (), FALSE, FALSE, 0);
 
   hbox = gtk_hbox_new (TRUE, 0);
-  gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 8);
+  gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-  messagewidget = gtk_label_new ("");
+  messagewidget = gtk_label_new (NULL);
   gtk_box_pack_start (GTK_BOX (hbox), messagewidget, TRUE, FALSE, 0);
 
-  scorewidget = gtk_label_new ("");
+  scorewidget = gtk_label_new (NULL);
   gtk_box_pack_start (GTK_BOX (hbox), scorewidget, TRUE, FALSE, 0);
 
   gtk_widget_show_all (application);
-
 }
 
 void
